#
#    Copyright (C) 2003-2005  James Tauber
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""
POSTing to the 'put' resource is intended as an alternative to PUTing a
resource in cases like HTML forms where you can't directly PUT.
"""


__provider__      = "PutProvider"
__contributions__ = {}    


import urllib

from leonardo.web_resource import PagePart

from leonardo.providers.page.page_provider import HTMLPage, ForbiddenPage

import put_templates

class PutProvider:
    
    
    def __init__(self, resource_manager, lfs, config):

        self.resource_manager = resource_manager
        self.lfs              = lfs
        self.config           = config

        self.put_path       = config.get("put", "put_path")

       
    def get(self, resource_id, request, main_resource):

        if resource_id != self.put_path:
            return None

        return ForbiddenPage(resource_id,  self.resource_manager, request, main_resource, self.config)

            
            
    def post(self, resource_id, request, main_resource):

        if resource_id != self.put_path:
            return None

        if not request.logged_in():
            return ForbiddenPage(resource_id,  self.resource_manager, request, main_resource, self.config)

        page_part = PutPagePart(resource_id, self.resource_manager, request, self.config)

        return HTMLPage(resource_id, page_part, self.resource_manager, request, main_resource, self.config)



class PutPagePart(PagePart):

    def __init__(self, resource_id, resource_manager, request, config):
        PagePart.__init__(self, resource_id)

        resource_to_put = request.fields.getvalue(config.get("put", "put_resource_field"))
        
        if resource_to_put:
           
            page = resource_manager.get(resource_to_put, request, self)

            if page.is_read_only():
                title = put_templates.put_read_only_title % locals()
                content = put_templates.put_read_only_content % locals()
    
            else:
                
                new_content =  request.fields.getvalue(config.get("put", "put_content_field"))
                new_content_type = request.fields.getvalue(config.get("put", "put_content_type_field"))
                
                page_title = request.fields.getvalue(config.get("editform", "edit_title_field"), None)
                keep_last_modified = request.fields.getvalue(config.get("editform", "keep_last_modified_field"), None)

                if keep_last_modified:
                    update_last_modified = False
                else:
                    update_last_modified = True

                page.set_property("page_title", page_title)
                page.set_property("author", request.get_user())

                for provider in resource_manager.get_contributors("post_processing"):
                    provider(request, page, None, config)                
                
                if page.exists():
                    replace = True
                else:
                    replace = False
                    
                page.set_content(new_content, new_content_type, update_last_modified)
                
                resource_link = config.get("DEFAULT", "cgi_root") + urllib.quote(resource_to_put)
                resource_label = resource_to_put
                resource_type = new_content_type
                
                if replace:
                    title = put_templates.put_replace_title % locals()
                    content = put_templates.put_replace_content % locals()
                else:
                    title = put_templates.put_create_title % locals()
                    content = put_templates.put_create_content % locals()

        else: # no resource_id provided
        
            title = put_templates.put_missing_id_title % locals()
            content = put_templates.put_missing_id_content % locals()            

        self.title = title
        self.html  = content
